#!/usr/bin/env python
import sys
import os
PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '.'))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)
import argparse
from forward_client.main import main as client_main
from forward_server.main import main as server_main

def main():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest='mode',help='running mode')

    server_mode = subparsers.add_parser('server',help='server mode')
    server_mode.add_argument('cfg_file',help='config file path')

    client_mode = subparsers.add_parser('client',help='client mode')
    client_mode.add_argument('cfg_file', help='config file path')
    #client_mode.add_argument('server_ip',help='server ip')
    #client_mode.add_argument('server_port',type=int,help='server port')

    args = parser.parse_args()
    if args.mode == 'server':
        server_main(args.cfg_file)
    elif args.mode == 'client':
        client_main(args.cfg_file)



if __name__=='__main__':
    sys.exit(main())


import sys
import os
import pyinotify
import ConfigParser

class PortMapper(pyinotify.ProcessEvent):
    class Coninfo(object):
        def __init__(self,ip,port):
            self.ip = ip
            self.port = port

    def __init__(self,map_file_path,call_back):
        # map out port to inner port,like:{1234:Coninfo('192.168.1.5',1234)}
        #self.__port_to_info = {1234:self.Coninfo('192.168.184.128',4321)}
        #self.__port_to_info = {1234:self.Coninfo('127.0.0.1',2222)}
        #self.__port_to_info = {1234:self.Coninfo('127.0.0.1',80),
        #                       1235:self.Coninfo('127.0.0.1',22),
        #                       1236: self.Coninfo('127.0.0.1',4321)}


        # map fileno to out port,like:{4:1234}
        self.__fileno_to_port = {}

        self.__map_file_path = map_file_path
        self.__callback = call_back
        self.__wm = pyinotify.WatchManager()
        self.__notifier = pyinotify.Notifier(self.__wm,self)
        self.__wm.add_watch(os.path.dirname(map_file_path), pyinotify.IN_MODIFY)
        self.__refresh_port_info()
        #self.__wm.watch_transient_file(map_file_path, pyinotify.IN_MODIFY, ProcessTransientFile)

    def __refresh_port_info(self):
        cf = ConfigParser.ConfigParser()
        temp_info = {}
        try:
            cf.readfp(open(self.__map_file_path, 'rb'))
            mappers = cf.items('MAPPER')
            for item in mappers:
                outer_port = int(item[0].lstrip().rstrip())
                inner_info = item[1].lstrip().rstrip().split(':')
                inner_ip = inner_info[0].lstrip().rstrip()
                inner_port = int(inner_info[1].lstrip().rstrip())
                temp_info[outer_port] = self.Coninfo(inner_ip,inner_port)
        except Exception,e:
            print e
            raise e
        self.__port_to_info = temp_info


    def watch_event(self,time_out_ms):
        try:
            self.__notifier.process_events()
            if self.__notifier.check_events(time_out_ms):
                self.__notifier.read_events()
                return True
            else:
                return False
        except Exception,e:
            print 'process_events exception'


    def process_IN_MODIFY(self, event):
        mod_file = os.path.join(event.path, event.name)
        if mod_file == self.__map_file_path:
            print  "Modify file: %s " % os.path.join(event.path, event.name)
            self.__refresh_port_info()
            self.__callback()

    def add_fileno_to_port(self,fileno,port):
        self.__fileno_to_port[fileno] = port

    def del_fileno_to_port(self,fileno):
        if self.__fileno_to_port.has_key(fileno):
            self.__fileno_to_port.pop(fileno)

    def get_outer_ports(self):
        return [port for port in self.__port_to_info]

    def get_inner_info_by_fileno(self,fileno):
        if not self.__fileno_to_port.has_key(fileno):
            return None,None

        out_port = self.__fileno_to_port[fileno]

        if not self.__port_to_info.has_key(out_port):
            return None,None

        return self.__port_to_info[out_port].ip,self.__port_to_info[out_port].port

    def get_inner_info_by_out_port(self,out_port):
        if self.__port_to_info.has_key(out_port):
            return self.__port_to_info[out_port].ip,self.__port_to_info[out_port].port
        else:
            return None,None


from enum import Enum
import select
from common import connector
from common import ring_buffer
import data_handler
import logging
logger = logging.getLogger('my_logger')

class InnerWorker(object):
    class State(Enum):
        NONE = 0
        WORKING = 1
        CLOSED = 2
        DONE = 3


    def __init__(self,worker_id,inner_socket):
        self.__worker_id = worker_id
        self.__state = self.State.NONE
        self.__connector = connector.Connector(inner_socket)
        self.__data_handler = data_handler.InnerDataHandler()
        self.__ring_buffer = ring_buffer.RingBuffer(1024 * 1024)
        if self.__connector.con_state == connector.CON_STATE.CON_CONNECTED:
            self.__state = self.State.WORKING

    def get_connector(self):
        return self.__connector

    def has_done(self):
        return self.__state == self.State.DONE

    def get_worker_id(self):
        return self.__worker_id

    def get_fileno(self):
        if self.__connector == None:
            return None

        return self.__connector.get_fileno()

    def handle_event(self,event):

        if self.__state == self.State.WORKING:
            self.__handle_working_event(event)

    def do_work(self,worker_manager):
        if self.__state == self.State.CLOSED:
            # close all paired outer worker
            out_workers = worker_manager.get_outer_workers_by_inner_worker_id(self.__worker_id)
            for ow in out_workers:
                ow.close()
            self.__state = self.State.DONE
            logger.debug("InnerWorker %d current state:CLOSED change state to DONE" % (self.__worker_id))
        elif self.__state == self.State.WORKING:
            if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.CLOSED
                logger.debug("InnerWorker %d current state:WORKING change state to CLOSED due connector state error:%s"%(self.__worker_id,str(self.__connector.con_state)) )
                return

        self.__data_handler.handle_data(self.__ring_buffer, worker_manager)
    def __handle_working_event(self, event):
        error_happen = False
        if event & select.EPOLLIN:
            recv_msg = self.__connector.recv()
            if len(recv_msg) > 0:
                # handle data
                self.__ring_buffer.put(bytearray(recv_msg))
                #self.__ring_buffer.print_buf()

            else:
                if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                    error_happen = True
                logger.error("InnerWorker %d current state:WORKING recv data error" % (self.__worker_id))

        elif event & select.EPOLLHUP:
            error_happen = True

        if error_happen:
            self.__connector.close()
            self.__state = self.State.CLOSED
            logger.debug("InnerWorker %d current state:WORKING change state to CLOSED" % (self.__worker_id))




class InnerWorkerManager(object):
    def __init__(self):
        self.__inner_map = {}


    def add_worker(self,worker):
        forward_id = worker.get_forward_id()
        self.__inner_map[forward_id] = worker

    def del_worker(self,forward_id):
        self.__inner_map.pop(forward_id)

    def get_worker(self,forward_id):
        if self.__inner_map.has_key(forward_id):
            return self.__inner_map[forward_id]
        else:
            return None

    def all_do(self,recever,outer_connector):
        for key,value in self.__inner_map.items():
            if value.has_done():
                self.__inner_map.pop(key)
                continue
            value.do_work(recever,outer_connector)

    def get_worker_by_fileno(self,fileno):
        for value in self.__inner_map.values():
            if value.get_fileno() == fileno:
                return value

        return None

    def close_all(self):
        for key, value in self.__inner_map.items():
            value.close()


import socket

class Acceptor(object):
    def __init__(self,ip,port):
        self.__ip = ip
        self.__port = port
        self.__init()

    def __init(self):
        self.__socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.__socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.__socket.bind((self.__ip, self.__port))
        self.__socket.listen(1)
        self.__socket.setblocking(0)

    def get_fileno(self):
        return self.__socket.fileno()

    def accept(self):
        return self.__socket.accept()

    def close(self):
        self.__socket.close()




class DATA_TYPE(object):
    NEW_CONNECTION = 0x00
    CONNECTION_SUCCESS = 0x01
    TRANS_DATA = 0x10
    CLOSE_CONNECTION = 0x20
    HEART_BEAT = 0x30


class ForwardData(object):
    def __init__(self,data_type=0,forward_id=0,inner_ip='',inner_port=0,data=None):
        self.data_type = data_type
        self.id = forward_id
        self.inner_ip = inner_ip
        self.inner_port = inner_port
        self.data = data


from pprint import pprint

class RingBuffer(object):
    def __init__(self,buffer_size):
        self.__buffer_size = buffer_size
        self.__buffer = bytearray(buffer_size)
        self.__read_ptr = 0
        self.__write_ptr = 0
        self.__data_len = 0

    def print_buf(self):
        current_buf = self.look(0,self.__data_len)
        print ' '.join(['0x%.2x' % x for x in current_buf])

    def buf_len(self):
        return self.__data_len

    def put(self,buf):
        buf_len = len(buf)
        if buf_len > self.__buffer_size - self.__data_len:
            raise Exception('input size:' + str(buf_len) + ' > writable size:' + str(self.__buffer_size - self.__data_len))
        if self.__buffer_size - self.__write_ptr >= buf_len:
            # write direct
            self.__buffer[self.__write_ptr:self.__write_ptr+buf_len] = buf
            self.__write_ptr += buf_len

        else:
            # front len
            front_len = self.__buffer_size - self.__write_ptr
            behind_len = buf_len - front_len
            self.__buffer[self.__write_ptr:self.__write_ptr+front_len] = buf[0:front_len]
            self.__buffer[0:behind_len] = buf[front_len:]
            self.__write_ptr = behind_len

        if self.__write_ptr >= self.__buffer_size:
            self.__write_ptr = 0
        self.__data_len += buf_len

    '''
        return and pop out the get_size len buf
    '''
    def get(self,get_size):
        if get_size > self.__data_len:
            raise Exception('get_size:' + str(get_size) + ' > data_len:' + str(self.__data_len))

        result_array = bytearray(get_size)
        if get_size <= self.__buffer_size - self.__read_ptr:
            result_array = self.__buffer[self.__read_ptr:self.__read_ptr + get_size]
            self.__read_ptr += get_size
        else:
            front_len = self.__buffer_size - self.__read_ptr
            behind_len = get_size - front_len
            result_array[0:front_len] = self.__buffer[self.__read_ptr:]
            result_array[front_len:] = self.__buffer[0:behind_len]
            self.__read_ptr = behind_len

        if self.__read_ptr >= self.__buffer_size:
            self.__read_ptr = 0

        self.__data_len -= get_size
        return result_array


    def look(self,from_location,look_size = 1):
        if look_size <= 0 or look_size > self.__data_len:
            return None
        elif look_size == 1:
            if self.__read_ptr + from_location >= self.__buffer_size:
                return self.__buffer[self.__read_ptr + from_location - self.__buffer_size ]
            else:
                return self.__buffer[self.__read_ptr + from_location]
        else:
            result_array = bytearray(look_size)
            if self.__read_ptr + from_location >= self.__buffer_size:
                return self.__buffer[self.__read_ptr + from_location - self.__buffer_size:self.__read_ptr + from_location - self.__buffer_size + look_size]
            else:
                if self.__read_ptr + from_location + look_size < self.__buffer_size:
                   return self.__buffer[self.__read_ptr + from_location:self.__read_ptr + from_location + look_size]
                else:
                    front_len = self.__buffer_size - self.__read_ptr - from_location
                    behind_len = look_size - front_len
                    result_array[0:front_len] = self.__buffer[self.__read_ptr:]
                    result_array[front_len:] = self.__buffer[0:behind_len]
                    return result_array

if __name__=='__main__':
    test_buffer = RingBuffer(10)
    buf1 = bytearray('1'*5)
    buf2 = bytearray('2'*7)

    test_buffer.put(buf1)
    test_buffer.print_buf()
    pprint(vars(test_buffer))

    get_buf1 = test_buffer.get(3)
    test_buffer.print_buf()
    pprint(vars(test_buffer))

    test_buffer.put(buf2)
    test_buffer.print_buf()
    pprint(vars(test_buffer))

    look1 = test_buffer.look(0,8)
    print look1
    look2 = test_buffer.look(2)
    print look2

    get_buf2 = test_buffer.get(8)
    test_buffer.print_buf()
    pprint(vars(test_buffer))




import sys
import os
PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '..'))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)
import inner_worker_manager
import outer_worker
from common import epoll_recever
import inner_worker
import argparse
import ConfigParser
import logging
from logging.handlers import RotatingFileHandler
logger = logging.getLogger('my_logger')

def log_config(level):
    if not os.path.isdir('/var/log/tcp_forward'):
        os.makedirs('/var/log/tcp_forward')
    logger.setLevel(logging._levelNames[level])
    handler = RotatingFileHandler("/var/log/tcp_forward/forward_client.log", maxBytes=10000000, backupCount=10)
    console = logging.StreamHandler()
    formatter = logging.Formatter(
        '%(asctime)s %(process)d %(levelname)s %(filename)s:%(lineno)s %(funcName)s [-] %(message)s ')
    handler.setFormatter(formatter)
    console.setFormatter(formatter)
    logger.addHandler(handler)
    logger.addHandler(console)

def main(cfg_file):
    cfg = ConfigParser.ConfigParser()
    cfg.readfp(open(cfg_file, 'rb'))
    log_config(cfg.get('DEFAULT','LOG_LEVEL'))
    logger.info("Process start!")

    outer_ip = cfg.get('DEFAULT','SERVER_IP')
    outer_port = int(cfg.get('DEFAULT','SERVER_PORT'))
    out_worker = outer_worker.OuterWorker(outer_ip, outer_port)
    recver = epoll_recever.Epoll_receiver()

    logger.info("Outer ip:%s:%d"%(outer_ip, outer_port))

    __inner_worker_manager = inner_worker_manager.InnerWorkerManager()

    while True:
        out_worker.do_work(recver,__inner_worker_manager)
        __inner_worker_manager.all_do(recver,out_worker.get_connector())

        events = recver.run()
        for fileno, event in events:
            #print 'get event:' + str(fileno)
            if fileno == out_worker.get_fileno():
                out_worker.handle_event(recver,event)
            else:
                __inner_worker = __inner_worker_manager.get_worker_by_fileno(fileno)
                if __inner_worker != None:
                    __inner_worker.handle_event(recver,event,out_worker.get_connector())
                else:
                    logger.error("Can't get fileno:%d inner_worker,unregister it"%(fileno))
                    epoll_recever.del_receiver(fileno)



if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    #parser.add_argument("server_ip", help="server ip")
    #parser.add_argument("server_port", help="server port")
    parser.add_argument("cfg_file", help="config file")

    args = parser.parse_args()

    sys.exit(main(args.cfg_file))


from enum import Enum
import select
from common import connector
from common import tools
import data_handler
import logging
logger = logging.getLogger('my_logger')

class OuterWorker(object):
    class State(Enum):
        NONE = 0
        CONNECTING_TO_INNER = 1
        WORKING = 2
        CLOSED = 3
        DONE = 4

    def __init__(self,worker_id,inner_ip,inner_port,outer_socket):
        self.__worker_id = worker_id
        self.__state = self.State.NONE
        self.__inner_ip = inner_ip
        self.__inner_port = inner_port
        self.__connector = connector.Connector(outer_socket)
        self.__data_handler = data_handler.OutDataHandler()

    def has_done(self):
        return self.__state == self.State.DONE

    def is_working(self):
        return self.__state == self.State.WORKING

    def handle_event(self,event,inner_connection):

        if self.__state == self.State.WORKING:
            self.__handle_working_event(event,inner_connection)

    def get_worker_id(self):
        return self.__worker_id

    def trans_data(self,msg):
        send_bytes = self.__connector.send(msg)
        if send_bytes <= 0:
            logger.error("OutWorker %d trans bytes <=0 change state to CLOSED",self.__worker_id)
            self.__state = self.State.CLOSED

    def close(self):
        if self.__state == self.State.DONE:
            return
        if self.__state == self.State.CONNECTING_TO_INNER:
            self.connecting_reply(False)
        else:
            self.__connector.close()
            self.__state = self.State.CLOSED
            logger.debug("OuterWorker %d current state:%s change state to CLOSED " % (self.__worker_id,str(self.__state)))

    def get_fileno(self):
        if self.__connector == None:
            return None

        return self.__connector.get_fileno()

    def do_work(self,inner_connection):

        if self.__state == self.State.NONE:
            if inner_connection and inner_connection.con_state == connector.CON_STATE.CON_CONNECTED:
                self.__data_handler.create_connection(self.__worker_id,self.__inner_ip,self.__inner_port,inner_connection)
                self.__state = self.State.CONNECTING_TO_INNER
                logger.debug("OuterWorker %d current state:NONE create inner connecttion to %s:%d ,change state to CONNECTING_TO_INNER"%
                             (self.__worker_id,self.__inner_ip,self.__inner_port))
            else:
                self.__connector.close()
                self.__state = self.State.DONE
                logger.debug("OuterWorker %d current state:NONE change state to DONE due inner_connection state error"%(self.__worker_id))
        elif self.__state == self.State.WORKING:
            if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.CLOSED
                logger.debug("OuterWorker %d current state:WORKING change state to CLOSED due connector state error:%s"%(self.__worker_id,str(self.__connector.con_state)) )

        elif self.__state == self.State.CLOSED:
            # send closed msg to remote
            if inner_connection and inner_connection.con_state == connector.CON_STATE.CON_CONNECTED:
                self.__data_handler.close_connection(self.__worker_id,self.__inner_ip,self.__inner_port,inner_connection)
            self.__connector.close()
            self.__state = self.State.DONE
            logger.debug("OuterWorker %d current state:CLOSED change state to DONE" %(self.__worker_id))

    def connecting_reply(self,connected):
        if self.__state != self.State.CONNECTING_TO_INNER:
            return

        if connected:
            self.__state = self.State.WORKING
            logger.debug("OuterWorker %d current state:CONNECTING_TO_INNER change state to WORKING" % (self.__worker_id))
        else:
            self.__connector.close()
            self.__state = self.State.DONE
            logger.debug("OuterWorker %d current state:CONNECTING_TO_INNER change state to DONE" % (self.__worker_id))

    def __handle_working_event(self, event,inner_connection):
        error_happen = False
        if event & select.EPOLLIN:
            recv_msg = self.__connector.recv()
            if len(recv_msg) > 0:
                # trans data
                #print 'recv outer %d msg:'%(self.__worker_id)
                #tools.print_hex_buf(recv_msg)
                try:
                    self.__data_handler.trans_data(self.__worker_id,self.__inner_ip,self.__inner_port,bytearray(recv_msg),inner_connection)
                except Exception,e:
                    error_happen = True
                    logger.error("OuterWorker %d current state:WORKING send data error" % (self.__worker_id))
            else:
                if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                    error_happen = True
                logger.error("OuterWorker %d current state:WORKING recv data error" % (self.__worker_id))

        elif event & select.EPOLLHUP:
            error_happen = True

        if error_happen:
            self.__connector.close()
            self.__state = self.State.CLOSED
            logger.debug("OuterWorker %d current state:WORKING change state to CLOSED" % (self.__worker_id))



import inner_worker
from common import forward_data, protocol_handler
from common import connector
from common.data_handler import DataHandler
import logging
logger = logging.getLogger('my_logger')


class OuterDataHandler(DataHandler):
    def handle_data(self,ring_buffer,inner_worker_manager):
        while True:
            parsed_data = super(OuterDataHandler,self).parse_data(ring_buffer)

            if parsed_data == None:
                return

            if parsed_data.data_type == forward_data.DATA_TYPE.NEW_CONNECTION:
                __inner_worker = inner_worker.InnerWorker(parsed_data.id,parsed_data.inner_ip,parsed_data.inner_port)
                inner_worker_manager.add_worker(__inner_worker)
            elif parsed_data.data_type == forward_data.DATA_TYPE.TRANS_DATA:
                # get the connector for the data_id
                __inner_worker = inner_worker_manager.get_worker(parsed_data.id)
                if __inner_worker != None and __inner_worker.is_working():
                    __inner_worker.trans_data(parsed_data.data)
            elif parsed_data.data_type == forward_data.DATA_TYPE.CLOSE_CONNECTION:
                __inner_worker = inner_worker_manager.get_worker(parsed_data.id)
                if __inner_worker != None:
                    __inner_worker.close()

    def send_heart_beat(self,outer_connector):
        forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.HEART_BEAT, 0,'0.0.0.0', 0, '')
        protocol_parser = protocol_handler.ProtocolHandler()
        send_package = protocol_parser.build_data(forw_data)
        if outer_connector and outer_connector.con_state == connector.CON_STATE.CON_CONNECTED:
            send_bytes = outer_connector.send(send_package)
            if send_bytes <= 0:
                logger.error("HeartBeat send failed")
                raise Exception("Send HeartBeat failed")
class InnerDataHandler(DataHandler):
    def tarns_data(self,forward_id,inner_ip,inner_port,data,outer_connector):

        ori = 0
        total_len = len(data)
        while ori < total_len:
            if total_len - ori <= 10000:
                send_data = data[ori:total_len]
            else:
                send_data = data[ori:ori+10000]

            forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.TRANS_DATA, forward_id,
                                             inner_ip, inner_port, send_data)

            protocol_parser = protocol_handler.ProtocolHandler()
            send_package = protocol_parser.build_data(forw_data)
            if outer_connector and outer_connector.con_state == connector.CON_STATE.CON_CONNECTED:
                send_bytes = outer_connector.send(send_package)
                if send_bytes <= 0:
                    logger.error("TransData send failed,forward_id:%d inner_ip:%s inner_port:%d"%(forward_id,inner_ip,inner_port))
                    raise Exception("Send data failed")
            ori += 10000

    def inner_closed(self,forward_id,inner_ip,inner_port,outer_connector):
        close_data = forward_data.ForwardData()
        close_data.id = forward_id
        close_data.data_type = forward_data.DATA_TYPE.CLOSE_CONNECTION
        close_data.inner_ip = inner_ip
        close_data.inner_port = inner_port
        close_data.data = ''

        protocol_parser = protocol_handler.ProtocolHandler()
        send_package = protocol_parser.build_data(close_data)
        if outer_connector and outer_connector.con_state == connector.CON_STATE.CON_CONNECTED:
            send_bytes = outer_connector.send(send_package)
            if send_bytes <= 0:
                logger.error("CloseData send failed,forward_id:%d inner_ip:%s inner_port:%d"%(forward_id,inner_ip,inner_port))

    def inner_connected(self,forward_id,inner_ip,inner_port,outer_connector):
        reply_data = forward_data.ForwardData(forward_data.DATA_TYPE.CONNECTION_SUCCESS, forward_id,
                                             inner_ip, inner_port, '')
        protocol_parser = protocol_handler.ProtocolHandler()
        send_package = protocol_parser.build_data(reply_data)
        if outer_connector and outer_connector.con_state == connector.CON_STATE.CON_CONNECTED:
            send_bytes = outer_connector.send(send_package)
            if send_bytes <= 0:
                logger.error("ConnectedData send failed,forward_id:%d inner_ip:%s inner_port:%d"%(forward_id,inner_ip,inner_port))


import random
import inner_worker
import outer_worker

class WorkerManager(object):
    def __init__(self):
        self.__mapper = {}
        self.__outer_workers = {}
        self.__inner_workers = {}
        self.__worker_id_seq = 0

    def generate_worker_id(self):
        self.__worker_id_seq += 1
        return self.__worker_id_seq

    def add_map(self,outer_worker_id,inner_worker_id):
        self.__mapper[outer_worker_id] = inner_worker_id

    def get_pair_inner_worker(self,outer_worker_id):
        if self.__mapper.has_key(outer_worker_id) and \
                self.__inner_workers.has_key(self.__mapper[outer_worker_id]):
            return self.__inner_workers[self.__mapper[outer_worker_id]]
        else:
            return None

    def add_outer_worker(self,outer_socket,inner_ip,inner_port):
        _worker_id = self.generate_worker_id()
        _paired_inner_worker = self.get_inner_worker_by_random()


        if not _paired_inner_worker :
            raise Exception('')
        _outer_worker = outer_worker.OuterWorker(_worker_id, inner_ip, inner_port, outer_socket)

        self.add_map(_worker_id, _paired_inner_worker.get_worker_id())
        self.__outer_workers[_worker_id] = _outer_worker
        return _outer_worker

    def add_inner_worker(self,inner_socket):

        _worker_id = self.generate_worker_id()
        _inner_worker = inner_worker.InnerWorker(_worker_id, inner_socket)

        self.__inner_workers[_worker_id] = _inner_worker

    def get_worker_by_id(self,worker_id):
        if self.__outer_workers.has_key(worker_id):
            return self.__outer_workers[worker_id]
        elif self.__inner_workers.has_key(worker_id):
            return self.__inner_workers[worker_id]
        else:
            return None

    def get_outer_worker_by_fileno(self,fileno):
        for worker in self.__outer_workers.values():
            if worker.get_fileno() == fileno:
                return worker
        return None

    def get_inner_worker_by_fileno(self,fileno):
        for worker in self.__inner_workers.values():
            if worker.get_fileno() == fileno:
                return worker
        return None

    def get_outer_workers_by_inner_worker_id(self,inner_id):
        result_workers = []
        for key,value in self.__mapper.items():
            if value == inner_id:
                if self.__outer_workers.has_key(key):
                    result_workers.append(self.__outer_workers[key])
        return result_workers

    def get_inner_worker_by_random(self):
        if len(self.__inner_workers) <= 0:
            return None

        _random_num = int(random.random()*100)%len(self.__inner_workers)
        return [value for value in self.__inner_workers.values()][_random_num]

    def remove_outer_worker(self,worker_id):
        self.__outer_workers.pop(worker_id)
        if self.__mapper.has_key(worker_id):
            self.__mapper.pop(worker_id)

    def remove_inner_worker(self,worker_id):
        self.__inner_workers.pop(worker_id)
        for key,value in self.__mapper.items():
            if value == worker_id:
                self.__mapper.pop(key)
    def all_worker_do(self):
        for id,worker in self.__inner_workers.items():
            if worker.has_done():
                self.remove_inner_worker(id)
                continue
            worker.do_work(self)

        for id,worker in self.__outer_workers.items():
            if worker.has_done():
                self.remove_outer_worker(id)
                continue
            _inner_worker = self.get_pair_inner_worker(worker.get_worker_id())
            if _inner_worker == None:
                worker.close()
                self.remove_outer_worker(id)
                continue
            worker.do_work(_inner_worker.get_connector())



import select

from enum import Enum

import data_handler
from common import connector
import logging
logger = logging.getLogger('my_logger')

class InnerWorker(object):
    class State(Enum):
        NONE = 0
        CONNECTING = 1
        WORKING = 2
        CLOSED = 3
        DONE = 4

    def __init__(self,forward_id,inner_ip,inner_port):
        self.__state = self.State.NONE
        self.__id = forward_id
        self.__inner_ip = inner_ip
        self.__inner_port = inner_port
        self.__connector = None
        self.__data_handler = data_handler.InnerDataHandler()

    def get_fileno(self):
        if self.__connector == None:
            return None

        return self.__connector.get_fileno()

    def get_forward_id(self):
        return self.__id

    def trans_data(self,msg):
        send_bytes = self.__connector.send(msg)
        if send_bytes <= 0:
            logger.error("InnerWorker %d trans bytes <=0 change state to CLOSED",self.__id)
            self.__state = self.State.CLOSED

    def close(self):
        if self.__state == self.State.DONE:
            return
        self.__connector.close()
        self.__state = self.State.CLOSED

    def has_done(self):
        return self.__state == self.State.DONE

    def is_working(self):
        return self.__state == self.State.WORKING

    def handle_event(self, recever,event,outer_connector):
        if self.__state == self.State.CONNECTING:
            self.__connector.refresh_con_state()
            if self.__connector.con_state == connector.CON_STATE.CON_CONNECTED:
                self.__data_handler.inner_connected(self.__id,self.__inner_ip,self.__inner_port,outer_connector)
                self.__state = self.State.WORKING
                recever.change_event(self.__connector.get_fileno(),select.EPOLLIN)
                logger.info("InnerWorker %d connect to %s:%d succed,change state to WORKING"%(self.__id,self.__inner_ip,self.__inner_port))
            elif self.__connector.con_state == connector.CON_STATE.CON_FAILED:
                self.__connector.close()
                self.__state = self.State.CLOSED
                logger.error("InnerWorker %d connect to %s:%d failed,change state to CLOSED"%(self.__id, self.__inner_ip, self.__inner_port))
        elif self.__state == self.State.WORKING:
            self.__handle_working_event(event,outer_connector)

    def do_work(self, recever,outer_connector):
        if self.__state == self.State.NONE:
            self.__connector = connector.ConnectorClient(self.__inner_ip, self.__inner_port)
            self.__connector.connect()

            if self.__connector.con_state == connector.CON_STATE.CON_CONNECTED:
                recever.add_receiver(self.__connector.get_fileno(), select.EPOLLIN)
                self.__state = self.State.WORKING
            elif self.__connector.con_state == connector.CON_STATE.CON_FAILED:
                self.__state = self.State.CLOSED
            elif self.__connector.con_state == connector.CON_STATE.CON_CONNECTING:
                recever.add_receiver(self.__connector.get_fileno(), select.EPOLLOUT)
                self.__state = self.State.CONNECTING
            logger.debug("InnerWorker %d connect to %s:%d current state:%s"%(self.__id,self.__inner_ip,self.__inner_port,str(self.__state)))
        elif self.__state == self.State.WORKING:
            if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.CLOSED
                logger.debug("InnerWorker %d current state:WORKING change state to DONE due connector state error:%s"%(self.__id,str(self.__connector.con_state)) )

        elif self.__state == self.State.CLOSED:
            #send closed message to outer
            self.__data_handler.inner_closed(self.__id,self.__inner_ip,self.__inner_port,outer_connector)
            recever.del_receiver(self.__connector.get_fileno())
            self.__state = self.State.DONE
            logger.debug("InnerWorker %d current state:CLOSED change state to DONE"%(self.__id))

    def __handle_working_event(self, event,outer_connector):
        error_happen = False
        if event & select.EPOLLIN:
            recv_msg = self.__connector.recv()
            if len(recv_msg) > 0:
                # trans data
                try:
                    self.__data_handler.tarns_data(self.__id, self.__inner_ip, self.__inner_port,
                                               bytearray(recv_msg), outer_connector)
                except Exception,e:
                    error_happen = True
                    logger.error("InnerWorker %d current state:WORKING send data error" % (self.__id))

            else:
                if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                    error_happen = True
                logger.error("InnerWorker %d current state:WORKING recv data error" % (self.__id))

        elif event & select.EPOLLHUP:
            error_happen = True

        if error_happen:
            self.__connector.close()
            self.__state = self.State.CLOSED
            logger.debug("InnerWorker %d current state:WORKING change state to CLOSED" % (self.__id))



from common.data_handler import DataHandler
from common.protocol_handler import ProtocolHandler
from common import forward_data
from common import connector
from common import tools
import logging
logger = logging.getLogger('my_logger')

class OutDataHandler(DataHandler):


    def create_connection(self,forward_id,inner_ip,inner_port,inner_connector):
        _protocol_handler = ProtocolHandler()
        forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.NEW_CONNECTION, forward_id,
                                             inner_ip, inner_port, '')
        send_package = _protocol_handler.build_data(forw_data)
        if inner_connector and inner_connector.con_state == connector.CON_STATE.CON_CONNECTED:
            send_bytes = inner_connector.send(send_package)
            if send_bytes <= 0:
                logger.error("CreateConnectionData send failed,forward_id:%d inner_ip:%s inner_port:%d" % (forward_id, inner_ip, inner_port))

    def trans_data(self,forward_id,inner_ip,inner_port,data,inner_connector):

        ori = 0
        total_len = len(data)
        while ori < total_len:
            if total_len - ori <= 10000:
                send_data = data[ori:total_len]
            else:
                send_data = data[ori:ori + 10000]

            _protocol_handler = ProtocolHandler()
            forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.TRANS_DATA, forward_id,
                                                 inner_ip, inner_port, send_data)
            send_package = _protocol_handler.build_data(forw_data)
            if inner_connector and inner_connector.con_state == connector.CON_STATE.CON_CONNECTED:
                send_bytes = inner_connector.send(send_package)
                if send_bytes <= 0:
                    logger.error("TransData to inner send failed,forward_id:%d inner_ip:%s inner_port:%d" % (forward_id, inner_ip, inner_port))
                    raise Exception("Send data failed")
                #print 'inner_connector send package'
                #tools.print_hex_buf(send_package)
            ori += 10000


    def close_connection(self,forward_id,inner_ip,inner_port,inner_connector):
        _protocol_handler = ProtocolHandler()
        forw_data = forward_data.ForwardData(forward_data.DATA_TYPE.CLOSE_CONNECTION, forward_id,
                                             inner_ip, inner_port, '')
        send_package = _protocol_handler.build_data(forw_data)
        if inner_connector and inner_connector.con_state == connector.CON_STATE.CON_CONNECTED:
            send_bytes = inner_connector.send(send_package)
            if send_bytes <= 0:
                logger.error("CloseConnectionData send failed,forward_id:%d inner_ip:%s inner_port:%d" % (forward_id, inner_ip, inner_port))

class InnerDataHandler(DataHandler):
    def handle_data(self, ring_buffer,worker_manager):
        while True:
            parsed_data = super(InnerDataHandler, self).parse_data(ring_buffer)
            if parsed_data == None:
                return

            if parsed_data.data_type == forward_data.DATA_TYPE.CONNECTION_SUCCESS:
                outer_worker = worker_manager.get_worker_by_id(parsed_data.id)
                if outer_worker != None:
                    outer_worker.connecting_reply(True)
            elif parsed_data.data_type == forward_data.DATA_TYPE.TRANS_DATA:
                outer_worker = worker_manager.get_worker_by_id(parsed_data.id)
                if outer_worker != None and outer_worker.is_working():
                    outer_worker.trans_data(parsed_data.data)
            elif parsed_data.data_type == forward_data.DATA_TYPE.CLOSE_CONNECTION:
                outer_worker = worker_manager.get_worker_by_id(parsed_data.id)
                if outer_worker != None:
                    outer_worker.close()
            elif parsed_data.data_type == forward_data.DATA_TYPE.HEART_BEAT:
                logger.debug('Recv heartbeat')


import forward_data, protocol_handler
import connector


class DataHandler(object):
    def __init__(self):
        pass

    def parse_data(self,ring_buffer):
        #get one complete package

        if ring_buffer.buf_len() <= 0:
            return None


        protocol_parse = protocol_handler.ProtocolHandler()
        package = None
        try:
            protocol_parse.del_wrong_header(ring_buffer)
            package = protocol_parse.get_one_complete_package(ring_buffer)
        except protocol_handler.ParseTimeout,e:
            print 'parse timeout,current buffer:'
            return None
        except protocol_handler.ParseError,e:
            print 'parse error'
            protocol_parse.pop_header(ring_buffer)
            return None
        except Exception,e:
            return None


        if package == None:
            print 'package is None'
            return None

        parsed_data = protocol_parse.parse_data(package)

        if parsed_data == None:
            print 'parse package failed'
            return None

        return parsed_data


import socket,select
import errno


class Epoll_receiver(object):
    def __init__(self):
        self.__recv_fds = {}
        self.__epoll = select.epoll()

    def add_receiver(self,fd,events):
        self.__recv_fds[fd] = events
        self.__epoll.register(fd, events)

    def change_event(self,fd,events):
        if self.__recv_fds.has_key(fd):
            self.__epoll.unregister(fd)
            self.__epoll.register(fd,events)

    def del_receiver(self,fd):
        if self.__recv_fds.has_key(fd):
            self.__recv_fds.pop(fd)
            self.__epoll.unregister(fd)

    def run(self):
        return self.__epoll.poll(1)


import select
import time
from enum import Enum

from common import connector, epoll_recever, ring_buffer
from data_handler import OuterDataHandler
import logging
from logging.handlers import RotatingFileHandler
logger = logging.getLogger('my_logger')

class OuterWorker(object):
    class State(Enum):
        NONE = 0
        CONNECTING = 1
        WORKING = 2
        DISCONNECTED = 3

    def __init__(self,outer_ip,outer_port):
        self.__state = self.State.NONE
        self.__outer_ip = outer_ip
        self.__outer_port = outer_port
        self.__connector = None
        self.__ring_buffer = ring_buffer.RingBuffer(10240 * 10240)
        self.__data_handler = OuterDataHandler()
        self.__last_heart_beat_time = int(time.time())

    def get_fileno(self):
        if self.__connector == None:
            return None

        return self.__connector.get_fileno()

    def get_connector(self):
        return self.__connector

    def send_heart_beat(self):
        try:
            self.__data_handler.send_heart_beat(self.__connector)
        except Exception,e:
            logger.error("OuterrWorker current state:WORKING send heartbeat error,change state to DISCONNECTED" )
            self.__state = self.State.DISCONNECTED

    def handle_event(self,recever,event):


        if self.__state == self.State.CONNECTING:
            self.__connector.refresh_con_state()

            if self.__connector.con_state == connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.WORKING
                recever.change_event(self.__connector.get_fileno(), select.EPOLLIN)
                logger.info("OuterWorker connect success change state to WORKING")

            elif self.__connector.con_state == connector.CON_STATE.CON_FAILED:
                self.__connector.close()
                self.__state = self.State.DISCONNECTED
                logger.error("OuterWorker connect failed,change state to DISCONNECTED")


        elif self.__state == self.State.WORKING:
            self.__handle_working_event(event)


    def do_work(self,recever,inner_worker_manager):


        if self.__state == self.State.NONE:
            #print 'outer_worker:connect to %s %d'%(self.__outer_ip,self.__outer_port)
            self.__connector = connector.ConnectorClient(self.__outer_ip, self.__outer_port)
            self.__connector.connect()

            if self.__connector.con_state == connector.CON_STATE.CON_CONNECTED:
                recever.add_receiver(self.__connector.get_fileno(), select.EPOLLIN)
                self.__state = self.State.WORKING
            elif self.__connector.con_state == connector.CON_STATE.CON_FAILED:
                #do nothing
                pass
            elif self.__connector.con_state == connector.CON_STATE.CON_CONNECTING:
                recever.add_receiver(self.__connector.get_fileno(), select.EPOLLOUT)
                self.__state = self.State.CONNECTING
        elif self.__state == self.State.WORKING:
            if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                self.__state = self.State.DISCONNECTED
                logger.debug("OuterWorker current state:WORKING change state to DONE due connector state error:%s"%(str(self.__connector.con_state)) )
                return
            else:
                current_time = time.time()
                if current_time - self.__last_heart_beat_time >= 30:
                    self.send_heart_beat()
                    self.__last_heart_beat_time = current_time
                    logger.debug("OuterWorker send hearbeat")
        elif self.__state == self.State.DISCONNECTED:
            inner_worker_manager.close_all()
            self.__state = self.State.NONE
            logger.debug("OuterWorker current state:DISCONNECTED close all inner_worker,change state to NONE")


        self.__data_handler.handle_data(self.__ring_buffer,inner_worker_manager)

    def __handle_working_event(self,event):
        #print '__handle_working_event'
        error_happen = False
        if event & select.EPOLLIN:
            recv_msg = self.__connector.recv()
            if len(recv_msg) > 0:
                #pass data
                self.__ring_buffer.put(bytearray(recv_msg))
                #self.__ring_buffer.print_buf()
            else:
                if self.__connector.con_state != connector.CON_STATE.CON_CONNECTED:
                    error_happen = True
                logger.error("OuterWorker current state:WORKING recv data error")

        elif event & select.EPOLLHUP:
            error_happen = True

        if error_happen:
            self.__connector.close()
            self.__state = self.State.DISCONNECTED
            logger.debug("OuterWorkercurrent state:WORKING change state to DISCONNECTED")


if __name__ == '__main__':
    out_worker = OuterWorker('192.168.184.1',1234)
    recver = epoll_recever.Epoll_receiver()

    while True:
        out_worker.do_work(recver,None)
        events = recver.run()
        for fileno, event in events:
            out_worker.handle_event(event)



import sys
import os
PROJECT_ROOT = os.path.abspath(os.path.join(
    os.path.dirname(os.path.realpath(__file__)), '..'))
if PROJECT_ROOT not in sys.path:
    sys.path.insert(0, PROJECT_ROOT)

from common import epoll_recever
from common import acceptor
import socket,select
import port_mapper
import worker_manager
import inner_worker
import outer_worker
import argparse
import ConfigParser
import logging
from logging.handlers import RotatingFileHandler

port_mapper_changed = False

logger = logging.getLogger('my_logger')


def build_outer_acceptors(out_acceptors,_port_mapper,recver):
    out_ports = _port_mapper.get_outer_ports()
    for port in out_ports:
        _out_acceptor = acceptor.Acceptor('0.0.0.0',port)
        filno = _out_acceptor.get_fileno()
        out_acceptors[filno] = _out_acceptor
        _port_mapper.add_fileno_to_port(filno,port)
        recver.add_receiver(filno,select.EPOLLIN)
        logger.info("Out port listen:" + str(port) + '->' +  str(_port_mapper.get_inner_info_by_out_port(port)))


def port_mapper_change_callback():
    global port_mapper_changed
    port_mapper_changed = True


def run(cfg):
    recver = epoll_recever.Epoll_receiver()

    # just one inner acceptor
    inner_port = int(cfg.get('DEFAULT','INNER_PORT'))
    inner_acceptor = acceptor.Acceptor('0.0.0.0', inner_port)
    recver.add_receiver(inner_acceptor.get_fileno(), select.EPOLLIN)
    logger.info('Inner port listen:' + str(inner_port))

    # there are many outer acceptor
    _port_mapper = port_mapper.PortMapper(cfg.get('DEFAULT','OUTER_PORTS_FILE'), port_mapper_change_callback)

    _worker_manager = worker_manager.WorkerManager()
    out_acceptors = {}
    build_outer_acceptors(out_acceptors, _port_mapper, recver)

    global port_mapper_changed
    while True:
        _port_mapper.watch_event(10)

        if port_mapper_changed:
            for acc_fileno, acc in out_acceptors.items():
                recver.del_receiver(acc_fileno)
                _port_mapper.del_fileno_to_port(acc_fileno)
                acc.close()
            out_acceptors.clear()
            logger.info('Port mapper changed,clear current outport listeners,recreating')
            build_outer_acceptors(out_acceptors, _port_mapper, recver)
            port_mapper_changed = False

        _worker_manager.all_worker_do()
        events = recver.run()

        for fileno, event in events:
            if fileno == inner_acceptor.get_fileno():
                # recv inner connection
                _inner_socket, address = inner_acceptor.accept()
                _inner_socket.setblocking(0)
                _worker_manager.add_inner_worker(_inner_socket)
                recver.add_receiver(_inner_socket.fileno(), select.EPOLLIN)
                logger.info('Inner socket accept')

            elif out_acceptors.has_key(fileno):
                # recv outer connection
                _outer_socket, address = out_acceptors[fileno].accept()
                _outer_socket.setblocking(0)
                _inner_ip, _inner_port = _port_mapper.get_inner_info_by_fileno(fileno)
                if not _inner_ip or not _inner_port:
                    _outer_socket.close()
                    continue
                try:
                    _outer_worker = _worker_manager.add_outer_worker(_outer_socket, _inner_ip, _inner_port)
                    recver.add_receiver(_outer_socket.fileno(), select.EPOLLIN)
                    logger.info("Outer socket accept, worker_id:%d fileno:%d to inner port:%s:%d"%
                                (_outer_worker.get_worker_id(),_outer_socket.fileno(),_inner_ip,_inner_port))
                except Exception, e:
                    _outer_socket.close()
                    continue
            else:
                _inner_worker = _worker_manager.get_inner_worker_by_fileno(fileno)
                if _inner_worker:
                    _inner_worker.handle_event(event)
                    continue

                _out_worker = _worker_manager.get_outer_worker_by_fileno(fileno)
                if _out_worker:
                    _inner_worker = _worker_manager.get_pair_inner_worker(_out_worker.get_worker_id())
                    if _inner_worker == None:
                        logger.error("OuterWorker %d get paired InnerWorker failed,remove"%(_out_worker.get_worker_id()))
                        _out_worker.close()
                        _worker_manager.remove_outer_worker(_out_worker.get_worker_id())
                        continue

                    _out_worker.handle_event(event, _inner_worker.get_connector())
                    continue


def log_config(level):
    if not os.path.isdir('/var/log/tcp_forward'):
        os.makedirs('/var/log/tcp_forward')
    logger.setLevel(logging._levelNames[level])
    handler = RotatingFileHandler("/var/log/tcp_forward/forward_server.log", maxBytes=10000000, backupCount=10)
    console = logging.StreamHandler()
    formatter = logging.Formatter('%(asctime)s %(process)d %(levelname)s %(filename)s:%(lineno)s %(funcName)s [-] %(message)s ')
    handler.setFormatter(formatter)
    console.setFormatter(formatter)
    logger.addHandler(handler)
    logger.addHandler(console)

def main(cfg_file):

    cfg = ConfigParser.ConfigParser()
    cfg.readfp(open(cfg_file, 'rb'))

    log_config(cfg.get('DEFAULT','LOG_LEVEL'))

    logger.info("Process start!")
    run(cfg)


if __name__=='__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("cfg_file", help="config file")

    args = parser.parse_args()

    sys.exit(main(args.cfg_file))